﻿// Coming Up Gadget
// by Eric Smith
// http://esmithy.net

// Depends on:
//  Outlook.js
//  Google.js
//  formatDate.js
//  Log.js

var UPDATE_INTERVAL_MILLIS = 15 * 1000;         // time between page flips
var ONE_DAY_MILLIS = 24 * 60 * 60 * 1000;		// milliseconds per day
var REFRESH_EVERY = 60;                         // number of page flips before a data refresh
var MAX_DAYS = 21;								// maximum number of days away for upcoming events
var MAX_EVENTS = 20;                            // maximum number of events to display
var EVENTS_PER_PAGE = 4;                        // number of events per page

var dataSource;				// Data source from which to retrieve events
var dataSourceIcon;			// image that indicates the data source
var today;                  // today's date, refreshed with every data refresh
var events;                 // array of events, limited by MAX_EVENTS and DATE_RANGE_MILLIS 
var pageIndex;              // index into the events array for the first event of the current page
var updateCounter = 0;      // counter incremented each time a new page is shown, and that causes data refresh when equal to REFRESH_EVERY
var intervalId;             // interval value that can be used to cancel updates

// Event object with fields matching the Outlook appointment item.
function Event(subject, entryId, start) {
	this.Subject = subject;
	this.EntryID = entryId;
	this.Start = start;
}

function isNull(obj) {
	return ((typeof obj === 'object') && !obj);
}

function showDate() {
	var dateElement = document.getElementById("Today");
	dateElement.innerHTML = "<img src='" + dataSourceIcon + "'/> &nbsp;" + today.formatDate("D j M Y");
}

function clearEvent(index) {

	var eventLink = document.getElementById("EventLink" + index);
	eventLink.innerHTML = "";

	var date = document.getElementById("Date" + index);
	date.innerHTML = "";

	var until = document.getElementById("Until" + index);
	until.innerHTML = "";
}

function clearEvents() {
	var i;
	for (i = 0; i < EVENTS_PER_PAGE; ++i) {
		clearEvent(i);
	}

	var range = document.getElementById("Range");
	range.innerHTML = "";
}

function showError(ex) {
	clearEvents();
	var events = document.getElementById("Events");
	events.className = "inactive";
	var error = document.getElementById("Error");
	error.className = "";
	var errorMsg = document.getElementById("ErrorMessage");
	errorMsg.innerHTML = ex.message;
}

function clearError() {
	var events = document.getElementById("Events");
	events.className = "";
	var error = document.getElementById("Error");
	error.className = "inactive";
}

function refreshToday() {
	var now = new Date();
	if (today === null || now.getDate() !== today.getDate()) {
		today = now;

		showDate();
	}
}

function showEvents() {

	if (typeof (events) === 'undefined') {
		return;
	}

	if (isNull(events) || events.length === 0) {
		showNoEvents();
		return;
	}

	clearEvents();

	var i;
	for (i = pageIndex; i < events.length && i - pageIndex < EVENTS_PER_PAGE; ++i) {
		showEvent(events[i], i - pageIndex);
	}

	showEventPageRanges(i);

	if (events.length > EVENTS_PER_PAGE) {
		if (i >= events.length) {
			pageIndex = 0;
		}
		else {
			pageIndex = i;
		}
	}
}

function refreshData() {
	try {
		events = dataSource.getEvents(MAX_DAYS, MAX_EVENTS);
		pageIndex = 0;
		showEvents();
	}
	catch (ex) {
		showError(ex);
	}
}

function update() {
	if (++updateCounter === REFRESH_EVERY) {
		// If we've done enough updates, refresh all the data.
		updateCounter = 0;
		refreshToday();
		refreshData();
	}
	else {
		// Otherwise, just show the next page of events.
		showEvents();
	}
}

function setupDataSource() {
	var source = System.Gadget.Settings.readString("CalendarSource");
	if (source === "Google") {
		dataSource = new Google();
		dataSource.username = System.Gadget.Settings.readString("Username");
		dataSource.password = System.Gadget.Settings.readString("Password");
		dataSourceIcon = "Google.png";
	}
	else {
		dataSource = new Outlook();
		dataSourceIcon = "Outlook.png";
	}
}

function loadMain() {
	clearError();
	setupDataSource();
	today = null;
	refreshToday();
	refreshData();
	if (!isNull(intervalId)) {
		clearInterval(intervalId);
	}
	intervalId = setInterval(update, UPDATE_INTERVAL_MILLIS);
}

function showEventPageRanges(currentPageIndex) {
    var range = document.getElementById("Range");
    var rangeHTML = "";
    var j, k;
    for (j = 0, k = 0; j <= events.length; ++j) {
        if ((j > 0 && j % EVENTS_PER_PAGE === 0) || j === events.length) {
            var css = (j === currentPageIndex) ? "strong" : "dim";
            rangeHTML += "<span class='" + css + "'>" + (k + 1) + "-" + j + "</span>&nbsp;&nbsp;&nbsp;&nbsp;";
            k = j;
        }
    }

    range.innerHTML = rangeHTML; //(pageIndex + 1) + "-" + i;
}

function showEvent(item, index) {
	var eventLink = document.getElementById("EventLink" + index);
	eventLink.innerHTML = item.Subject;
	eventLink.title = item.Subject;
	eventLink.href = item.EntryID;

	var date = document.getElementById("Date" + index);
	var startDate = new Date(item.Start);
	date.innerHTML = startDate.formatDate("D j M");

	var until = document.getElementById("Until" + index);
	var days = getDaysUntil(today, startDate);
	if (days === 0)
	{
		until.innerHTML = "Today";
	}
	else if (days === 1)
	{
		until.innerHTML = "1 day";
	}
	else
	{
		until.innerHTML = days + " days";
	}
}

function openEvent() {
    var element = event.srcElement;
	var id = element.href;
	dataSource.openEvent(id);
}

function showNoEvents() {
    clearEvents();
    var eventLink = document.getElementById("EventLink0");
    eventLink.innerHTML = "Nothing coming up...";
}

function getDaysUntil(today, date) {
    return Math.round(((date.getTime() - today.getTime()) / ONE_DAY_MILLIS) + 0.5);
}

